#include "MainWindow.h"
#include "ui_MainWindow.h"

MainWindow::MainWindow(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::MainWindow),
	device(0)
{
	ui->setupUi(this);
	device = new UdpDevice;

	connect(device, &DeviceProxy::portOpened, [this]() { showMessage(device->portName() + " is opened."); });
	connect(device, &DeviceProxy::portClosed, [this]() { showMessage(device->portName() + " is closed."); });
	connect(device, &DeviceProxy::errorOccurred, [this](QString err) { showError(device->portName() + " error :\n\t" + err); });
	connect(device, &DeviceProxy::readyRead, this, &MainWindow::readResponse, Qt::QueuedConnection);
	connect(device, &DeviceProxy::infoMessage, [this](QString mess) { showMessage(mess); });

	connect(ui->pushButtonOpen, &QPushButton::clicked, this, &MainWindow::openPort);
	connect(ui->pushButtonClose, &QPushButton::clicked, device, &UdpDevice::close);
	connect(ui->pushButtonSend, &QPushButton::clicked, this, &MainWindow::sendCommand);
}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::showMessage(QString mess)
{
	ui->textEdit->append(mess);
}

void MainWindow::showError(QString mess)
{
	showMessage(mess.prepend("<font color=red>").append("</font>"));
}

void MainWindow::showSended(QString mess)
{
	showMessage(mess.prepend("<font color=green> Sended : ").append("</font>"));
}

void MainWindow::showReceived(QString mess)
{
	showMessage(mess.prepend("<font color=blue>Received : ").append("</font>"));
}

void MainWindow::openPort()
{
	if(ui->lineEditPort->text().isEmpty() || !device)
		showError("Empty port name");
	else
		QMetaObject::invokeMethod(device, "initiate", Qt::QueuedConnection, Q_ARG(QString, ""), Q_ARG(int, ui->lineEditPort->text().toInt()));
}

void MainWindow::sendCommand()
{
	if(device && device->isOpen() && !ui->lineEditCommand->text().isEmpty())
	{

		if(device->write(ui->lineEditCommand->text().toLocal8Bit()) > 0)
		{
			showSended(ui->lineEditCommand->text());
			ui->lineEditCommand->clear();
		}
		else
			showError("Failed to send command : " + device->errorString());
	}
	else
		showError("Unable to send command");
}

void MainWindow::readResponse()
{
	if(device)
		showReceived(device->readAll());
}
